library example.basic_auth.server;

import 'dart:async';
import 'package:mongo_dart/mongo_dart.dart';
import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_session_mongo/jaguar_session_mongo.dart';
import 'package:jaguar_session/jaguar_session.dart';
import 'package:jaguar_mongo/jaguar_mongo.dart';

import '../../common/models/book/book.dart';
import '../../common/user.dart';
import '../../common/books.dart';

//This is the session manager that is passed to SessionInterceptor
MgoCookieSession sessionManager(Db db) => new MgoCookieSession(db);

const String mongoUrl = 'mongodb://localhost:27018/example';

class BaseApi {
  MongoDb mongo(Context ctx) => new MongoDb(mongoUrl);

  BasicAuth basicAuth(Context ctx) {
    Db db = ctx.getInput(MongoDb);
    return new BasicAuth(kModelManager, sessionManager: sessionManager(db));
  }

  Authorizer authorizer(Context ctx) {
    Db db = ctx.getInput(MongoDb);
    return new Authorizer(kModelManager, sessionManager: sessionManager(db));
  }
}

/// This route group contains login and logout authentication routes
@Api()
@WrapOne(#mongo)
//You must provide an instance of SessionManager through sessionManager parameter
class AuthRoutes extends BaseApi {
  @Post(path: '/login')
  // Wrap [BasicAuth] interceptor to authenticate
  @WrapOne(#basicAuth)
  void login(Context ctx) {}

  @Post(path: '/logout')
  @WrapOne(#authorizer)
  Future<Null> logout(Context ctx) async {
    final sesMan = ctx.getVariable<SessionManager>();
    await sesMan.deleteSession();
  }
}

/// Collection of routes that need authorization
@Api(path: '/book')
// Wrap [Authorizer] interceptor to authorize requests
@Wrap(const [#mongo, #authorizer])
class StudentRoutes extends BaseApi {
  @Get(path: '/all')
  Response<String> getAllBooks(Context ctx) {
    // Authorized user can be obtained from Authorizer as Input
    User user = ctx.getInput(Authorizer);
    print(user.username);
    List<Map> ret = books.values.map((Book book) => book.toMap()).toList();
    return Response.json(ret);
  }

  @Get(path: '/:id')
  Response<String> getBook(Context ctx) {
    final String id = ctx.pathParams['id'];
    final Book book = books[id];
    return Response.json(book.toMap());
  }
}

@Api(path: '/api')
class LibraryApi {
  @IncludeApi()
  final AuthRoutes auth = new AuthRoutes();

  @IncludeApi()
  final StudentRoutes student = new StudentRoutes();
}

server() async {
  final jaguar = new Jaguar();
  jaguar.addApiReflected(new LibraryApi());

  await jaguar.serve();
}
